import time

from django.contrib.auth.models import User
from django.http import JsonResponse
from django.utils import timezone
from django.views.decorators.csrf import csrf_exempt

from projects.models import Comment, Issue


def time_decorator(view):
    """時間を計測するデコレーター"""

    def _inner(*args, **kwargs):
        start = time.time()
        response = view(*args, **kwargs)
        end = time.time()
        print(f"計測時間: {end - start}")
        return response

    return _inner


@time_decorator
def comments_with_issue(request):
    """課題つきコメント一覧"""
    contents = []

    comments = Comment.objects.all()
    # comments = Comment.objects.select_related("issue").all()
    for comment in comments:
        contents.append(
            {
                "comment.content": comment.content,
                "issue.content": comment.issue.content,
            }
        )

    return JsonResponse({"contents": contents})


@time_decorator
def comments_by_user(request):
    """ユーザーごとのコメント一覧"""
    contents = []

    users = User.objects.all()
    # users = User.objects.prefetch_related(
    #     "comment_set"
    # ).all()
    for user in users:
        comments = user.comment_set.all()
        for comment in comments:
            contents.append(
                {
                    "name": user.username,
                    "content": comment.content,
                }
            )

    return JsonResponse({"contents": contents})


@csrf_exempt
@time_decorator
def import_issues(request):
    """課題の一括作成"""
    posted_issue_contents = request.POST[
        "contents"
    ].split(",")
    posted_user_id = request.POST["user_id"]

    for content in posted_issue_contents:
        Issue.objects.create(user_id=posted_user_id,
                             content=content)
    # issues = [
    #     Issue(user_id=posted_user_id, content=content)
    #     for content in posted_issue_contents
    # ]
    # Issue.objects.bulk_create(issues, batch_size=2)

    return JsonResponse({})


@csrf_exempt
@time_decorator
def update_issues(request):
    """課題の一括更新"""
    issues = Issue.objects.all()

    for issue in issues:
        if (
            issue.created_at.date()
            == timezone.now().date()
        ):
            issue.content = f"(today):{issue.content}"
        else:
            issue.content = f"(past):{issue.content}"
        issue.save()

    # issues_updated = []
    # for issue in issues:
    #     if issue.created_at.date() == timezone.now().date():
    #         issue.content = f"(today):{issue.content}"
    #     else:
    #         issue.content = f"(past):{issue.content}"
    #     issues_updated.append(issue)
    # Issue.objects.bulk_update(issues_updated, ["content"])

    return JsonResponse({})
